################################################################
### ----- C score matrix calculation and vizualization ----- ###
#################### ----- v 1.3 ----- #########################
## ----- made by Ondrej Mottl -- ondrej.mottl@gmail.com ----- ##
################################################################

##########################
### ----- SETUP ----- ####
##########################
library(EcoSimR)
library(readxl)
library(reshape2)
library(ggplot2)

#######################################
### ----- function definition ----- ###
#######################################

SES.count<-function (data.source)
{
  observed.dataset<-data.source$Obs
  random.dataset<-data.source$Sim
  x<-(observed.dataset-mean(random.dataset))/sd(random.dataset)
  return(x)
}

c.matrix<-function (data.source, N.species = FALSE, randomization = 50000)
{
  require(EcoSimR)
  data.source<- as.data.frame(data.source)
  row.names(data.source) <- data.source[,1]
  data.order <- data.source[order(-rowSums(data.source[,2:length(data.source)])),]
  
  
  if(N.species!=FALSE)
  {
    data.work <- data.order[c(1:N.species),]
  }
  else
  {
    data.work<-data.order
    N.species <- nrow(data.work)
  }
  
  res.mat<-matrix(nrow=N.species,ncol = N.species)
  rownames(res.mat)<-row.names(data.work)
  colnames(res.mat)<-row.names(data.work)
  res.mat.p <-res.mat
  comb<-t(combn(1:N.species,2))
  N.comb <- nrow(comb)
  res.list <-vector(length = N.comb)
  res.p<-vector(length = N.comb)
  
  for (i in 1:N.comb)
  {
    print(paste0("species combination ",i," of ",N.comb))
    data.select <- data.work[c(comb[i,1],comb[i,2]),]
    data.select.small<-data.select[,]
    data.select.small.df<-as.data.frame(data.select.small)
    c.res.small<-cooc_null_model(data.select.small.df, algo = "sim2", metric = "c_score",nReps = randomization, saveSeed = FALSE, burn_in = randomization) #50 000
    res.list[i]<-SES.count(c.res.small)
    alpha.p <- 1-(1-0.05)**(1/(nrow(comb))) #Sidak correction
    
    if (isFALSE(is.na(res.list[i])))
    {
      if (res.list[i] < 0)
      {
        res.p[i]<-(quantile(c.res.small$Sim, alpha.p/2 ) > c.res.small$Obs) 
      }
      else
      {
        res.p[i]<-(quantile(c.res.small$Sim, 1-(alpha.p/2) ) <c.res.small$Obs) # 
      }
      res.mat[comb[i,1],comb[i,2]]<-res.list[i]
      res.mat.p[comb[i,1],comb[i,2]]<-res.p[i]
      print("run-OK")  
    }
    else
    {
      print("run-NA")
    }
    
  }
  result.list<-vector("list", 2)
  result.list[[1]]<-res.mat
  result.list[[2]]<-res.mat.p
  print("DONE")
  return(result.list)
}

plot.hist <- function (data.source)
{
  require(ggplot2)
  df <- data.frame(Sim=data.source$Sim) 
  
  ggplot(data=df,aes(x=Sim))+
    geom_density(color="black",fill="gray")+
    geom_vline(xintercept = mean(data.source$Sim),color= "black",lty=3,size=1)+
    geom_vline(xintercept = data.source$Obs,color= "red",lty=1,size=1 )+
    theme_bw()+
    labs(y="Frequency",x="C-score",title = "Distribution of simulated C-scores")
}


plot.matrix <- function (data.source, main.title="NA")
{
  require(ggplot2)
  require(reshape2)
  
  Species.list<- row.names(data.source[[1]])
  melt.1<-melt(data.source[[1]])
  melt.2<-melt(data.source[[2]])
  melt.1$SIG<-factor(melt.2$value, levels = c("TRUE","FALSE"),labels = c("TRUE","FALSE"))
  melt.1.a<-na.omit(melt.1)
  melt.1.b<-melt.1.a[melt.1.a$Var1!=melt.1.a$Var2,]
  
  Pallete.1 <- c (1,16)
  names(Pallete.1) <- c("FALSE","TRUE")
  
  g0<- ggplot(melt.1.b, aes(Var2, Var1)) + 
    theme_bw()+
    scale_x_discrete(drop=FALSE,limits = Species.list)+
    scale_y_discrete(drop=FALSE,limits = Species.list)+
    theme(axis.text.x = element_text(angle = 90, hjust = 1),panel.grid.major = element_line(colour = "white"))+
    coord_fixed() 
  
  g1 <- g0 +
    labs(y="Species",x="Species", fill="SES",title = main.title)+
    scale_fill_gradient2(low = "green", high = "red", mid = "white",midpoint = 0,limit = c(-10,10))+
    scale_shape_manual("Significant",values=Pallete.1, drop=FALSE)
  
  g2 <- g0 + 
    labs(y="Species",x="Species", fill="SES",title = "Significance test")
  
  
  if(nrow(melt.1.b)>0)
  {g3<-g1 + geom_tile(aes(fill = value), color="white")+
    geom_point(aes(shape=SIG), size=3)}
  else
  {g3<-g1 + geom_tile(aes(fill = 0), color="white")}
  
  print(g3)
}

###########################
### ----- Example ----- ###
###########################
treexants<-read.csv("treexants.csv")

treexants$`ANON.002`<-treexants$`ANON.002`+treexants$`ANON.004`
treexants<-treexants[,c(1:8,10:186)]
treexants<-subset(treexants, treexants$`N.nest`>0)
dataN <- subset(treexants, Forest=="Primary_N")
dataN<-dataN[,1:169]
dataW <- subset(treexants, Forest=="Primary_W")
dataW<-dataW[,1:169]
dataY <- subset(treexants, Forest=="Primary_Y")
dataY<-dataY[,1:169]


# load your own data in the same structure
dataW.t <- as.data.frame(t(dataW[,2:length(dataW)]))
dataW.t <-data.frame(row.names(dataW.t),dataW.t)

dataN.t <- as.data.frame(t(dataN[,2:length(dataN)]))
dataN.t <-data.frame(row.names(dataN.t),dataN.t)

dataY.t <- as.data.frame(t(dataY[,2:length(dataY)]))
dataY.t <-data.frame(row.names(dataY.t),dataY.t)

# set number of randomization 50 000 is recomended
N.Run <- 10000

# calculate pairwise C-score for 10 most common species. 
# For all species calculation change to N.species=FALSE
dataW.res2<-c.matrix(dataW.t,N.species = 10 , randomization = N.Run)
dataN.res2<-c.matrix(dataN.t,N.species = 10 , randomization = N.Run)
dataY.res2<-c.matrix(dataY.t,N.species = 10 , randomization = N.Run)

# Plot result
wanangplot2.all <- plot.matrix(dataW.res2,"200 m")
numbaplot2.all <- plot.matrix(dataN.res2,"900 m") 
yawanplot2.all <- plot.matrix(dataY.res2,"1800 m")

multiplot(wanangplot2.all,numbaplot2.all,yawanplot2.all , cols=1)

# better way to plot multiple figures
library(ggpubr)
ggarrange(wanangplot2.all,numbaplot2.all,yawanplot2.all , ncol=1,nrow=3, legend="right",common.legend = T)


# extract values
extractTable <- function(data.source)
{
  one <- melt(data.source[[1]])
  two <- melt(data.source[[2]])
  fin<- data.frame(one, two$value)
  fin<-na.omit(fin)
  names(fin)<-c("Species A","Species B","SES","Significant")
  return(fin)
}

C.score.pair.W<- extractTable(dataW.res2)
write.csv(C.score.pair.W, file="C.score.pair.W.csv")
C.score.pair.N<- extractTable(dataN.res2)
write.csv(C.score.pair.N, file="C.score.pair.N.csv")
C.score.pair.Y<- extractTable(dataY.res2)
write.csv(C.score.pair.Y, file="C.score.pair.Y.csv")

